@@ -22,6 +22,7 @@ from UM.Logger import Logger
 from UM.Preferences import Preferences
 from UM.View.Renderer import Renderer #For typing.
 from UM.OutputDevice.OutputDeviceManager import OutputDeviceManager
+from UM.Workspace.WorkspaceMetadataStorage import WorkspaceMetadataStorage
 from UM.i18n import i18nCatalog
 from UM.Version import Version
 
@@ -100,9 +101,14 @@ class Application:
 
         self._app_install_dir = self.getInstallPrefix()  # type: str
 
+        self._workspace_metadata_storage = WorkspaceMetadataStorage()  # type: WorkspaceMetadataStorage
+
     def getAPIVersion(self) -> "Version":
         return self._api_version
 
+    def getWorkspaceMetadataStorage(self) -> WorkspaceMetadataStorage:
+        return self._workspace_metadata_storage
+
     # Adds the command line options that can be parsed by the command line parser.
     # Can be overridden to add additional command line options to the parser.
     def addCommandLineOptions(self) -> None:
@@ -131,6 +137,10 @@ class Application:
 
     # Performs initialization that must be done before start.
     def initialize(self) -> None:
+        Logger.log("d", "Initializing %s", self._app_display_name)
+        Logger.log("d", "App Version %s", self._version)
+        Logger.log("d", "Api Version %s", self._api_version)
+        Logger.log("d", "Build type %s", self._build_type or "None")
         # For Ubuntu Unity this makes Qt use its own menu bar rather than pass it on to Unity.
         os.putenv("UBUNTU_MENUPROXY", "0")
 
@@ -166,9 +176,9 @@ class Application:
         self._controller = Controller(self)
         self._output_device_manager = OutputDeviceManager()
 
-        self._operation_stack = OperationStack(self._controller) # type: OperationStack
+        self._operation_stack = OperationStack(self._controller)
 
-        self._plugin_registry = PluginRegistry(self)  #type: PluginRegistry
+        self._plugin_registry = PluginRegistry(self)
 
         self._plugin_registry.addPluginLocation(os.path.join(self._app_install_dir, "lib", "uranium"))
         self._plugin_registry.addPluginLocation(os.path.join(self._app_install_dir, "lib64", "uranium"))
@@ -217,7 +227,7 @@ class Application:
         self.startSplashWindowPhase()
         self.startPostSplashWindowPhase()
 
-    def getContainerRegistry(self):
+    def getContainerRegistry(self) -> ContainerRegistry:
         return self._container_registry
 
     ##  Get the lock filename
@@ -398,13 +408,19 @@ class Application:
     def getExtensions(self) -> List["Extension"]:
         return self._extensions
 
+    # Returns the path to the folder of the app itself, e.g.: '/root/blah/programs/Cura'.
     @staticmethod
-    def getInstallPrefix() -> str:
+    def getAppFolderPrefix() -> str:
         if "python" in os.path.basename(sys.executable):
             executable = sys.argv[0]
         else:
             executable = sys.executable
-        return os.path.abspath(os.path.join(os.path.dirname(os.path.realpath(executable)), ".."))
+        return os.path.dirname(os.path.realpath(executable))
+
+    # Returns the path to the folder the app is installed _in_, e.g.: '/root/blah/programs'
+    @staticmethod
+    def getInstallPrefix() -> str:
+        return os.path.abspath(os.path.join(Application.getAppFolderPrefix(), ".."))
 
     __instance = None   # type: Application
 
