@@ -80,10 +80,10 @@ class Backend(PluginObject):
             return
         Logger.log("i", "Started engine process: %s", self.getEngineCommand()[0])
         self._backendLog(bytes("Calling engine with: %s\n" % self.getEngineCommand(), "utf-8"))
-        t = threading.Thread(target = self._storeOutputToLogThread, args = (self._process.stdout,))
+        t = threading.Thread(target = self._storeOutputToLogThread, args = (self._process.stdout,), name = "EngineOutputThread")
         t.daemon = True
         t.start()
-        t = threading.Thread(target = self._storeStderrToLogThread, args = (self._process.stderr,))
+        t = threading.Thread(target = self._storeStderrToLogThread, args = (self._process.stderr,), name = "EngineErrorThread")
         t.daemon = True
         t.start()
 
@@ -123,7 +123,9 @@ class Backend(PluginObject):
             kwargs["startupinfo"] = su
             kwargs["creationflags"] = 0x00004000  # BELOW_NORMAL_PRIORITY_CLASS
         try:
-            return subprocess.Popen(command_list, stdin = subprocess.DEVNULL, stdout = subprocess.PIPE, stderr = subprocess.PIPE, **kwargs)
+            # STDIN needs to be None because we provide no input, but communicate via a local socket instead. The NUL device sometimes doesn't exist on some computers.
+            # STDOUT and STDERR need to be pipes because we'd like to log the output on those channels into the application log.
+            return subprocess.Popen(command_list, stdin = None, stdout = subprocess.PIPE, stderr = subprocess.PIPE, **kwargs)
         except PermissionError:
             Logger.log("e", "Couldn't start back-end: No permission to execute process.")
         except FileNotFoundError:
