@@ -38,6 +38,7 @@ class Controller:
         self._views = {}  # type: Dict[str, View]
 
         self._active_tool = None  # type: Optional[Tool]
+        self._fallback_tool = "TranslateTool"  # type: str
         self._tool_operation_active = False
         self._tools = {}  # type: Dict[str, Tool]
         self._camera_tool = None #type: Optional[Tool]
@@ -55,10 +56,9 @@ class Controller:
         PluginRegistry.addType("input_device", self.addInputDevice)
 
     ##  Add a view by name if it"s not already added.
-    #   \param name \type{string} Unique identifier of view (usually the plugin name)
     #   \param view \type{View} The view to be added
     def addView(self, view: View) -> None:
-        name = view.getPluginId()
+        name = view.getId()
         if name not in self._views:
             self._views[name] = view
             view.setRenderer(self._application.getRenderer())
@@ -67,7 +67,6 @@ class Controller:
             Logger.log("w", "%s was already added to view list. Unable to add it again.", name)
 
     ##  Request view by name. Returns None if no view is found.
-    #   \param name \type{string} Unique identifier of view (usually the plugin name)
     #   \return View \type{View} if name was found, none otherwise.
     def getView(self, name: str) -> Optional[View]:
         try:
@@ -115,7 +114,7 @@ class Controller:
     #   \param name \type{string} Unique identifier of stage (usually the plugin name)
     #   \param stage \type{Stage} The stage to be added
     def addStage(self, stage: Stage) -> None:
-        name = stage.getPluginId()
+        name = stage.getId()
         if name not in self._stages:
             self._stages[name] = stage
             self.stagesChanged.emit()
@@ -169,7 +168,7 @@ class Controller:
     ##  Add an input device (e.g. mouse, keyboard, etc) if it's not already added.
     #   \param device The input device to be added
     def addInputDevice(self, device: InputDevice) -> None:
-        name = device.getPluginId()
+        name = device.getId()
         if name not in self._input_devices:
             self._input_devices[name] = device
             device.event.connect(self.event)
@@ -194,7 +193,18 @@ class Controller:
             self._input_devices[name].event.disconnect(self.event)
             del self._input_devices[name]
 
-    ##  Request tool by name. Returns None if no view is found.
+    ##  Request the current fallbacl tool.
+    #   \return Id of the fallback tool
+    def getFallbackTool(self) -> str:
+        return self._fallback_tool
+
+    ##  Set the current active tool. The tool must be set by name.
+    #   \param tool The tools name which shall be used as fallback
+    def setFallbackTool(self, tool: str) -> None:
+        if self._fallback_tool is not tool:
+            self._fallback_tool = tool
+
+    ##  Request tool by name. Returns None if no tool is found.
     #   \param name \type{string} Unique identifier of tool (usually the plugin name)
     #   \return tool \type{Tool} if name was found, None otherwise.
     def getTool(self, name: str) -> Optional["Tool"]:
@@ -212,7 +222,7 @@ class Controller:
     ##  Add a Tool (transform object, translate object) if its not already added.
     #   \param tool \type{Tool} Tool to be added
     def addTool(self, tool: "Tool") -> None:
-        name = tool.getPluginId()
+        name = tool.getId()
         if name not in self._tools:
             self._tools[name] = tool
             tool.operationStarted.connect(self._onToolOperationStarted)
@@ -264,12 +274,12 @@ class Controller:
 
         from UM.Scene.Selection import Selection  # Imported here to prevent a circular dependency.
         if not self._active_tool and Selection.getCount() > 0:  # If something is selected, a tool must always be active.
-            if "TranslateTool" in self._tools:
-                self._active_tool = self._tools["TranslateTool"]  # Then default to the translation tool.
+            if self._fallback_tool in self._tools:
+                self._active_tool = self._tools[self._fallback_tool]  # Then default to the translation tool.
                 self._active_tool.event(ToolEvent(ToolEvent.ToolActivateEvent))
                 tool_changed = True
             else:
-                Logger.log("w", "Controller does not have an active tool and could not default to Translate tool.")
+                Logger.log("w", "Controller does not have an active tool and could not default to the tool, called \"{}\".".format(self._fallback_tool))
 
         if tool_changed:
             Selection.setFaceSelectMode(False)
@@ -410,13 +420,12 @@ class Controller:
             return
         camera.setZoomFactor(camera.getDefaultZoomFactor())
         self._camera_tool.setOrigin(Vector(0, 100, 0))  # type: ignore
+        self.setCameraOrigin(coordinate)
         if coordinate == "home":
             camera.setPosition(Vector(0, 100, 700))
-            camera.lookAt(Vector(0, 100, 0))
             self._camera_tool.rotateCamera(0, 0)  # type: ignore
         elif coordinate == "3d":
             camera.setPosition(Vector(-750, 600, 700))
-            camera.lookAt(Vector(0, 100, 100))
             self._camera_tool.rotateCamera(0, 0)  # type: ignore
         else:
             # for comparison is == used, because might not store them at the same location
@@ -424,19 +433,57 @@ class Controller:
 
             if coordinate == "x":
                 camera.setPosition(Vector(0, 100, 700))
-                camera.lookAt(Vector(0, 100, 0))
                 self._camera_tool.rotateCamera(angle, 0)  # type: ignore
             elif coordinate == "y":
                 if angle == 90:
                     # Prepare the camera for top view, so no rotation has to be applied after setting the top view.
                     camera.setPosition(Vector(0, 100, 100))
-                    camera.lookAt(Vector(0, 100, 0))
                     self._camera_tool.rotateCamera(90, 0)  # type: ignore
                     # Actually set the top view.
                     camera.setPosition(Vector(0, 800, 1))
+                    self.setCameraOrigin("z")
                     camera.lookAt(Vector(0, 100, 1))
                     self._camera_tool.rotateCamera(0, 0)  # type: ignore
                 else:
                     camera.setPosition(Vector(0, 100, 700))
-                    camera.lookAt(Vector(0, 100, 0))
                     self._camera_tool.rotateCamera(0, angle)  # type: ignore
+
+    # Position camera view according to defined position
+    def setCameraPosition(self, x_position: int = 0, y_position: int = 0, z_position: int = 0) -> None:
+        camera = self._scene.getActiveCamera()
+        if not camera:
+            return
+        camera.setPosition(Vector(x_position, y_position, z_position))
+
+    # Indicate position where the camera should point at
+    def setLookAtPosition(self, x_look_at_position: int = 0, y_look_at_position: int = 0, z_look_at_position: int = 0) -> None:
+        camera = self._scene.getActiveCamera()
+        if not camera:
+            return
+        camera.lookAt(Vector(x_look_at_position, y_look_at_position, z_look_at_position))
+
+    # Sets the zoom factor of the camera
+    def setCameraZoomFactor(self, camera_zoom_factor: float = 0) -> None:
+        camera = self._scene.getActiveCamera()
+        if not camera:
+            return
+        camera.setZoomFactor(camera_zoom_factor)
+
+    ##  Changes the origin of the camera, i.e. where it looks at.
+    #   \param coordinate One of the following options:
+    #    - "home": The centre of the build plate.
+    #    - "3d": The centre of the build volume.
+    #    - "x", "y" and "z": Also the centre of the build plate. These are just
+    #      aliases for the setCameraRotation function.
+    def setCameraOrigin(self, coordinate: str = "home"):
+        camera = self._scene.getActiveCamera()
+        if not camera:
+            return
+        coordinates = {
+            "home": Vector(0, 100, 0),
+            "3d": Vector(0, 100, 100),
+            "x": Vector(0, 100, 0),
+            "y": Vector(0, 100, 0),
+            "z": Vector(0, 100, 1)
+        }
+        camera.lookAt(coordinates[coordinate])
