@@ -1,6 +1,9 @@
-# Copyright (c) 2016 Ultimaker B.V.
+# Copyright (c) 2019 Ultimaker B.V.
 # Uranium is released under the terms of the LGPLv3 or higher.
 
+from typing import Dict, Any
+
+
 ##  Helper functions for dealing with Python dictionaries.
 
 ##  Find the key corresponding to a certain value
@@ -11,7 +14,7 @@
 #   \return The key matching to value. Note that if the dictionary contains multiple instances of value it is undefined which exact key is returned.
 #
 #   \exception ValueError is raised when the value is not found in the dictionary.
-def findKey(dictionary, search_value):
+def findKey(dictionary: Dict[Any, Any], search_value: Any) -> Any:
     for key, value in dictionary.items():
         if value == search_value:
             return key
