@@ -1,5 +1,5 @@
 # Copyright (c) 2016 Ultimaker B.V.
-# Uranium is released under the terms of the AGPLv3 or higher.
+# Uranium is released under the terms of the LGPLv3 or higher.
 
 ##  \file Event.py
 #   Contains the Event class and important subclasses used throughout UM.
@@ -7,7 +7,7 @@
 
 ##  Base event class.
 #   Defines the most basic interface for events and several constants to identify event types.
-from typing import List, Any, Callable
+from typing import List, Any, Callable, Optional
 
 
 class Event:
@@ -50,13 +50,13 @@ class MouseEvent(Event):
     #   \param last_x The X coordinate of the previous mouse event. Can be None. It is used to calculate deltaX.
     #   \param last_y The Y coordinate of the previous mouse event. Cam be None. It is used to calculate deltaY.
     #   \param buttons The buttons that are associated with this event.
-    def __init__(self, event_type: int, x: int = 0, y: int = 0, last_x: int = None, last_y: int = None, buttons: List = None) -> None: #pylint: disable=bad-whitespace
+    def __init__(self, event_type: int, x: int = 0, y: int = 0, last_x: int = None, last_y: int = None, buttons: List[str] = None) -> None: #pylint: disable=bad-whitespace
         super().__init__(event_type)
         self._x = x
         self._y = y
         self._last_x = last_x
         self._last_y = last_y
-        self._buttons = []  # type: List
+        self._buttons = []  # type: List[str]
         if buttons:
             self._buttons = buttons
 
@@ -72,12 +72,12 @@ class MouseEvent(Event):
 
     ##  The X coordinate of the previous event.
     @property
-    def lastX(self) -> int:
+    def lastX(self) -> Optional[int]:
         return self._last_x
 
     ##  The Y coordinate of the previous event.
     @property
-    def lastY(self) -> int:
+    def lastY(self) -> Optional[int]:
         return self._last_y
 
     ##  The change in X position between this event and the previous event.
@@ -98,7 +98,7 @@ class MouseEvent(Event):
 
     ##  The list of buttons associated with this event.
     @property
-    def buttons(self) -> List:
+    def buttons(self) -> List[str]:
         return self._buttons
 
 
@@ -179,7 +179,7 @@ class ToolEvent(Event):
 
 ##  Event used to call a function.
 class CallFunctionEvent(Event):
-    def __init__(self, func: Callable, args: Any, kwargs: Any) -> None:
+    def __init__(self, func: Callable[..., Any], args: Any, kwargs: Any) -> None:
         super().__init__(Event.CallFunctionEvent)
         self._function = func
         self._args = args
