@@ -1,35 +1,36 @@
 # Copyright (c) 2018 Ultimaker B.V.
 # Uranium is released under the terms of the LGPLv3 or higher.
 
-from typing import Any, Dict, List, Optional, Union, TYPE_CHECKING
+from typing import Any, Dict, List, Optional, Union, TYPE_CHECKING, cast
 
-from UM.PluginRegistry import PluginRegistry
-from UM.Logger import Logger
 from .FileReader import FileReader
 from .FileWriter import FileWriter
 from PyQt5.QtCore import QObject, pyqtProperty, pyqtSlot, QUrl
 
+from UM.Logger import Logger
 from UM.Platform import Platform
+from UM.PluginRegistry import PluginRegistry
 
 from UM.i18n import i18nCatalog
 i18n_catalog = i18nCatalog("uranium")
 
 if TYPE_CHECKING:
     from UM.Qt.QtApplication import QtApplication
-    from UM.Mesh.MeshReader import MeshReader
-    from UM.Mesh.MeshWriter import MeshWriter
 
 ##  Central class for reading and writing meshes.
 #   This class is created by Application and handles reading and writing mesh files.
 class FileHandler(QObject):
-    _instance = None    # type: FileHandler
-    _application = None # type: QtApplication
 
-    def __init__(self, writer_type: str = "unknown_file_writer", reader_type: str = "unknown_file_reader", parent: QObject = None) -> None:
+    def __init__(self, application: "QtApplication", writer_type: str = "unknown_file_writer", reader_type: str = "unknown_file_reader", parent: QObject = None) -> None:
+        if cast(FileHandler, self.__class__).__instance is not None:
+            raise RuntimeError("Try to create singleton '%s' more than once" % self.__class__.__name__)
+        cast(FileHandler, self.__class__).__instance = self
+
         super().__init__(parent)
 
-        self._readers = {} # type: Dict[str, MeshReader]
-        self._writers = {} # type: Dict[str, MeshWriter]
+        self._application = application
+        self._readers = {} # type: Dict[str, FileReader]
+        self._writers = {} # type: Dict[str, FileWriter]
 
         self._writer_type = writer_type # type: str
         self._reader_type = reader_type # type: str
@@ -37,22 +38,6 @@ class FileHandler(QObject):
         PluginRegistry.addType(self._writer_type, self.addWriter)
         PluginRegistry.addType(self._reader_type, self.addReader)
 
-    @classmethod
-    def setApplication(cls, application: "QtApplication") -> None:
-        cls._application = application
-
-    @classmethod
-    def getApplication(cls) -> "QtApplication":
-        return cls._application
-
-    ##  Return the singleton instance of the file handler.
-    @classmethod
-    def getInstance(cls, *args: Any, **kwargs: Any) -> "FileHandler":
-        if not cls._instance:
-            cls._instance = cls()
-
-        return cls._instance
-
     @pyqtProperty("QStringList", constant = True)
     def supportedReadFileTypes(self) -> List[str]:
         file_types = []
@@ -104,12 +89,14 @@ class FileHandler(QObject):
                 description = output.get("description", ext)
                 mime_type = output.get("mime_type", "text/plain")
                 mode = output.get("mode", FileWriter.OutputMode.TextMode)
+                hide_in_file_dialog = output.get("hide_in_file_dialog", False)
                 supported_types.append({
                     "id": entry["id"],
                     "extension": ext,
                     "description": description,
                     "mime_type": mime_type,
-                    "mode": mode
+                    "mode": mode,
+                    "hide_in_file_dialog": hide_in_file_dialog,
                 })
         return supported_types
 
@@ -127,10 +114,10 @@ class FileHandler(QObject):
                         supported_types[ext] = description
         return supported_types
 
-    def addReader(self, reader: "MeshReader") -> None:
+    def addReader(self, reader: "FileReader") -> None:
         self._readers[reader.getPluginId()] = reader
 
-    def addWriter(self, writer: "MeshWriter") -> None:
+    def addWriter(self, writer: "FileWriter") -> None:
         self._writers[writer.getPluginId()] = writer
 
     # Try to read the data from a file using a specified Reader.
@@ -138,15 +125,15 @@ class FileHandler(QObject):
     # \param file_name The name of the file to load.
     # \param kwargs Keyword arguments.
     # \returns None if nothing was found
-    def readerRead(self, reader: FileReader, file_name: str, **kwargs: Any):
+    def readerRead(self, reader, file_name: str, **kwargs: Any):
         raise NotImplementedError("readerRead must be implemented by subclasses.")
 
     ##  Get a mesh writer object that supports writing the specified mime type
     #
     #   \param mime The mime type that should be supported.
-    #   \return A MeshWriter instance or None if no mesh writer supports the specified mime type. If there are multiple
+    #   \return A FileWriter instance or None if no mesh writer supports the specified mime type. If there are multiple
     #           writers that support the specified mime type, the first entry is returned.
-    def getWriterByMimeType(self, mime: str) -> Optional["MeshWriter"]:
+    def getWriterByMimeType(self, mime: str) -> Optional["FileWriter"]:
         writer_data = PluginRegistry.getInstance().getAllMetaData(filter={self._writer_type: {}}, active_only=True)
         for entry in writer_data:
             for output in entry[self._writer_type].get("output", []):
@@ -156,7 +143,7 @@ class FileHandler(QObject):
         return None
 
     ##  Get an instance of a mesh writer by ID
-    def getWriter(self, writer_id: str) -> Optional["MeshWriter"]:
+    def getWriter(self, writer_id: str) -> Optional["FileWriter"]:
         if writer_id not in self._writers:
             return None
 
@@ -165,7 +152,7 @@ class FileHandler(QObject):
     ##  Find a Reader that accepts the given file name.
     #   \param file_name The name of file to load.
     #   \returns Reader that accepts the given file name. If no acceptable Reader is found None is returned.
-    def getReaderForFile(self, file_name: str) -> Optional["MeshReader"]:
+    def getReaderForFile(self, file_name: str) -> Optional["FileReader"]:
         for id, reader in self._readers.items():
             try:
                 if reader.acceptsFile(file_name):
@@ -173,4 +160,10 @@ class FileHandler(QObject):
             except Exception as e:
                 Logger.log("e", str(e))
 
-        return None
\ No newline at end of file
+        return None
+
+    __instance = None   # type: FileHandler
+
+    @classmethod
+    def getInstance(cls, *args, **kwargs) -> "FileHandler":
+        return cls.__instance
