@@ -5,11 +5,10 @@ import os
 import threading
 from contextlib import contextmanager
 import functools
-from typing import List
+from typing import List, Callable, Any
 
 from PyQt5.QtCore import pyqtSlot as pyqt5PyqtSlot
 from UM.Logger import Logger
-
 # A simple profiler which produces data suitable for viewing as a flame graph
 # when using the Big Flame Graph plugin.
 #
@@ -19,11 +18,14 @@ from UM.Logger import Logger
 # Set the environment variable URANIUM_FLAME_PROFILER to something before
 # starting the application to make the profiling code available.
 
-def enabled():
+
+def enabled() -> bool:
     return "URANIUM_FLAME_PROFILER" in os.environ
 
+
 record_profile = False  # Flag to keep track of whether we are recording data.
 
+
 # Profiling data is build up of a tree of these kinds of nodes. Each node
 # has a name, start time, end time, and a list of children nodes which are
 # other functions/methods which were called by this function.
@@ -80,6 +82,7 @@ clear_profile_requested = False
 record_profile_requested = False
 stop_record_profile_requested = False
 
+
 ##  Fetch the accumulated profile data.
 #
 #   \return \type{ProfileCallNode} or None if there is no data.
@@ -93,11 +96,13 @@ def getProfileData():
     fill_children = _fillInProfileSpaces(start_time, end_time, raw_profile_calls)
     return _ProfileCallNode("", 0, start_time, end_time, fill_children)
 
+
 ##  Erase any profile data.
 def clearProfileData():
     global clear_profile_requested
     clear_profile_requested = True
 
+
 ##  Start recording profile data.
 def startRecordingProfileData():
     global record_profile_requested
@@ -105,11 +110,13 @@ def startRecordingProfileData():
     stop_record_profile_requested = False
     record_profile_requested = True
 
+
 ##  Stop recording profile data.
 def stopRecordingProfileData():
     global stop_record_profile_requested
     stop_record_profile_requested = True
 
+
 def _fillInProfileSpaces(start_time, end_time, profile_call_list):
     result = []
     time_counter = start_time
@@ -124,9 +131,11 @@ def _fillInProfileSpaces(start_time, end_time, profile_call_list):
 
     return result
 
+
 def secondsToMS(value):
     return math.floor(value *1000)
 
+
 ##  Profile a block of code.
 #
 #   Use this context manager to wrap and profile a block of code.
@@ -147,13 +156,15 @@ def profileCall(name):
     else:
         yield
 
+
 ##  Return whether we are recording profiling information.
 #
 #   \return \type{bool} True if we are recording.
-def isRecordingProfile():
+def isRecordingProfile() -> bool:
     global record_profile
     return record_profile and threading.main_thread() is threading.current_thread()
 
+
 def updateProfileConfig():
     global child_accu_stack
     global record_profile
@@ -177,6 +188,7 @@ def updateProfileConfig():
             record_profile = False
             Logger.log('d', 'Stopping record stop_record_profile_requested')
 
+
 ##  Decorator which can be manually applied to methods to record profiling information.
 #
 #   \type{Callable}
@@ -193,10 +205,11 @@ def profile(function):
     else:
         return function
 
+
 ##  Drop in replacement for PyQt5's pyqtSlot decorator which records profiling information.
 #
 #   See the PyQt5 documentation for information about pyqtSlot.
-def pyqtSlot(*args, **kwargs):
+def pyqtSlot(*args, **kwargs) -> Callable[..., Any]:
     if enabled():
         def wrapIt(function):
             @functools.wraps(function)
