@@ -144,12 +144,12 @@ class Matrix:
     ##  Return the transpose of the matrix.
     def getTransposed(self) -> "Matrix":
         try:
-            return Matrix(numpy.transpose(self._data))
+            return Matrix(self._data.transpose())
         except:
             return Matrix(self._data)
 
     def transpose(self) -> None:
-        self._data = numpy.transpose(self._data)
+        self._data = self._data.transpose()
 
     ##  Translate the matrix based on Vector.
     #   \param direction The vector by which the matrix needs to be translated.
@@ -400,7 +400,7 @@ class Matrix:
         self._data[2, 3] = -1.
         self._data[3, 2] = (2. * far * near) / (near - far)
 
-    def decompose(self):
+    def decompose(self) -> Tuple[Vector, "Matrix", Vector, Vector]:
         '''
         SOURCE: https://github.com/matthew-brett/transforms3d/blob/e402e56686648d9a88aa048068333b41daa69d1a/transforms3d/affines.py
         Decompose 4x4 homogenous affine matrix into parts.
@@ -459,37 +459,39 @@ class Matrix:
         syz = sy_syz / sy
         # Reconstruct rotation matrix, ensure positive determinant
         Rmat = numpy.array([M0, M1, M2]).T
-        if numpy.linalg.det(Rmat) < 0:
-            sx *= -1
-            Rmat[:, 0] *= -1
+
+        # The original code ensures that the determinant is positive, but I can't find a single situation where this
+        # is actualy used / needed by us. It is, however, one of the more expensive parts of this function.
+        #if numpy.linalg.det(Rmat) < 0:
+        #    sx *= -1
+        #    Rmat[:, 0] *= -1
 
         return Vector(data = T), Matrix(data=Rmat), Vector(data = numpy.array([sx, sy, sz])), Vector(data=numpy.array([sxy, sxz, syz]))
 
     def _unitVector(self, data: numpy.array, axis: Optional[int] = None, out: Optional[numpy.array] = None) -> numpy.array:
         """Return ndarray normalized by length, i.e. Euclidean norm, along axis.
-
+        >>> matrix = Matrix()
         >>> v0 = numpy.random.random(3)
-        >>> v1 = unit_vector(v0)
+        >>> v1 = matrix._unitVector(v0)
         >>> numpy.allclose(v1, v0 / numpy.linalg.norm(v0))
         True
         >>> v0 = numpy.random.rand(5, 4, 3)
-        >>> v1 = unit_vector(v0, axis=-1)
-        >>> v2 = v0 / numpy.expand_dims(numpy.sqrt(numpy.sum(v0*v0, axis=2)), 2)
+        >>> v1 = matrix._unitVector(v0, axis=-1)
+        >>> v2 = v0 / numpy.expand_dims(numpy.sqrt(numpy.sum(v0 * v0, axis=2)), 2)
         >>> numpy.allclose(v1, v2)
         True
-        >>> v1 = unit_vector(v0, axis=1)
-        >>> v2 = v0 / numpy.expand_dims(numpy.sqrt(numpy.sum(v0*v0, axis=1)), 1)
+        >>> v1 = matrix._unitVector(v0, axis=1)
+        >>> v2 = v0 / numpy.expand_dims(numpy.sqrt(numpy.sum(v0 * v0, axis=1)), 1)
         >>> numpy.allclose(v1, v2)
         True
         >>> v1 = numpy.empty((5, 4, 3))
-        >>> unit_vector(v0, axis=1, out=v1)
+        >>> matrix._unitVector(v0, axis=1, out=v1)
         >>> numpy.allclose(v1, v2)
         True
-        >>> list(unit_vector([]))
+        >>> list(matrix._unitVector([]))
         []
-        >>> list(unit_vector([1]))
+        >>> list(matrix._unitVector([1]))
         [1.0]
-
         """
         if out is None:
             data = numpy.array(data, dtype = numpy.float64, copy = True)
