@@ -16,6 +16,7 @@ from UM.Logger import Logger
 from UM.Message import Message
 from UM.MimeTypeDatabase import MimeTypeDatabase  # To get the type of container we're loading.
 from UM.Resources import Resources
+from UM.Signal import Signal
 from UM.Version import Version as UMVersion
 
 catalog = i18nCatalog("uranium")
@@ -64,12 +65,14 @@ class PackageManager(QObject):
         self._installed_package_dict = {}  # type: Dict[str, Dict[str, Any]] # A dict of all installed packages
         self._to_remove_package_set = set()  # type: Set[str] # A set of packages that need to be removed at the next start
         self._to_install_package_dict = {}  # type: Dict[str, Dict[str, Any]]  # A dict of packages that need to be installed at the next start
+        self._dismissed_packages = set()    # type: Set[str] # A set of packages that are dismissed by the user
 
         # There can be plugins that provide remote packages (and thus, newer / different versions for a package).
         self._available_package_versions = {}  # type: Dict[str, Set[UMVersion]]
 
         self._packages_with_update_available = set()  # type: Set[str]
 
+    packageInstalled = Signal()  # Emits the package_id (str) of an installed package
     installedPackagesChanged = pyqtSignal()  # Emitted whenever the installed packages collection have been changed.
     packagesWithUpdateChanged = pyqtSignal()
 
@@ -97,6 +100,25 @@ class PackageManager(QObject):
         self._packages_with_update_available = packages
         self.packagesWithUpdateChanged.emit()
 
+    def isPackageCompatible(self, package_api_version: UMVersion) -> bool:
+        """
+        Check whether an API version is compatible with the application's API
+        version.
+        :param package_api_version: The API version to check.
+        :return: ``True`` if packages with this API version are compatible, or
+        ``False`` if they are not.
+        """
+        app_api_version = self._application.getAPIVersion()
+
+        if app_api_version.getMajor() != package_api_version.getMajor():
+            return False
+
+        # minor versions are backwards compatible
+        if app_api_version.getMinor() < package_api_version.getMinor():
+            return False
+
+        return True
+
     def checkIfPackageCanUpdate(self, package_id: str) -> bool:
         available_versions = self._available_package_versions.get(package_id)
 
@@ -153,6 +175,7 @@ class PackageManager(QObject):
                     self._installed_package_dict = management_dict.get("installed", {})
                     self._to_remove_package_set = set(management_dict.get("to_remove", []))
                     self._to_install_package_dict = management_dict.get("to_install", {})
+                    self._dismissed_packages = set(management_dict.get("dismissed", []))
                     Logger.log("i", "Loaded user packages management file from %s", self._user_package_management_file_path)
             except FileNotFoundError:
                 Logger.log("i", "User package management file %s doesn't exist, do nothing", self._user_package_management_file_path)
@@ -219,7 +242,8 @@ class PackageManager(QObject):
                 data_dict = {"version": PackageManager.Version,
                              "installed": self._installed_package_dict,
                              "to_remove": list(self._to_remove_package_set),
-                             "to_install": self._to_install_package_dict}
+                             "to_install": self._to_install_package_dict,
+                             "dismissed": list(self._dismissed_packages)}
                 json.dump(data_dict, f, sort_keys = True, indent = 4)
                 Logger.log("i", "Package management file %s was saved", self._user_package_management_file_path)
 
@@ -296,6 +320,10 @@ class PackageManager(QObject):
 
         return all_installed_ids
 
+    # Get a list of packages that the user has installed.
+    def getUserInstalledPackages(self) -> List[str]:
+        return [package for package in self._installed_package_dict]
+
     ## Get a list of tuples that contain the package ID and version.
     #  Used by the Marketplace to check which packages have updates available.
     def getAllInstalledPackageIdsAndVersions(self) -> List[Tuple[str, str]]:
@@ -338,26 +366,50 @@ class PackageManager(QObject):
     def getToRemovePackageIDs(self) -> Set[str]:
         return self._to_remove_package_set
 
+    def dismissAllIncompatiblePackages(self, incompatible_packages: List[str]) -> None:
+        self._dismissed_packages.update(incompatible_packages)
+        self._saveManagementData()
+        Logger.debug("Dismissed Incompatible package(s): {}".format(incompatible_packages))
+
+    def getDismissedPackages(self) -> List[str]:
+        return list(self._dismissed_packages)
+
+    def reEvaluateDismissedPackages(self, subscribed_packages_payload: List[Dict[str, Any]], sdk_version: str) -> None:
+        """
+        It removes a package from the "dismissed incompatible packages" list, if
+        it gets updated in the meantime. We check every package from the payload
+        against our current Cura SDK version, and if it is in there - we remove
+        the already dismissed package from the above mentioned list.
+        :param subscribed_packages_payload: The response from Web Cura, a list
+        of packages that a user is subscribed to.
+        :param sdk_version: Current Cura SDK version.
+        """
+        dismissed_packages = self.getDismissedPackages()
+        if dismissed_packages:
+            for package in subscribed_packages_payload:
+                if package["package_id"] in dismissed_packages and sdk_version in package["sdk_versions"]:
+                    self.removeFromDismissedPackages(package["package_id"])
+
+    def removeFromDismissedPackages(self, package: str) -> None:
+        if package in self._dismissed_packages:
+            self._dismissed_packages.remove(package)
+            Logger.debug("Removed package [%s] from the dismissed packages list" % package)
+
     # Checks if the given package is installed (at all).
     def isPackageInstalled(self, package_id: str) -> bool:
         return self.getInstalledPackageInfo(package_id) is not None
 
-    # This is called by drag-and-dropping curapackage files.
-    @pyqtSlot(QUrl)
-    def installPackageViaDragAndDrop(self, file_url: str) -> None:
-        filename = QUrl(file_url).toLocalFile()
-        return self.installPackage(filename)
-
-    # Schedules the given package file to be installed upon the next start.
+    ## Schedules the given package file to be installed upon the next start.
+    # \return The to-be-installed package_id or None if something went wrong
     @pyqtSlot(str)
-    def installPackage(self, filename: str) -> None:
+    def installPackage(self, filename: str) -> Optional[str]:
         has_changes = False
         package_id = ""
         try:
             # Get package information
             package_info = self.getPackageInfo(filename)
             if not package_info:
-                return
+                return None
             package_id = package_info["package_id"]
 
             # If the package is being installed but it is in the list on to remove, then it is deleted from that list.
@@ -398,6 +450,12 @@ class PackageManager(QObject):
                         self._packages_with_update_available.remove(package_id)
                         self.packagesWithUpdateChanged.emit()
 
+        if has_changes:
+            self.packageInstalled.emit(package_id)
+            return package_id
+        else:
+            return None
+
     # Schedules the given package to be removed upon the next start.
     # \param package_id id of the package
     # \param force_add is used when updating. In that case you actually want to uninstall & install
