@@ -7,10 +7,12 @@ import os.path
 import re
 import shutil
 import tempfile
-from typing import Dict, Generator, List, Optional, cast
+from typing import Dict, Generator, List, Optional, Union, cast
 
 from UM.Logger import Logger
 from UM.Platform import Platform
+from UM.Version import Version
+
 
 class ResourceTypeError(Exception):
     pass
@@ -417,7 +419,6 @@ class Resources:
         if cls.ApplicationVersion == "master" or cls.ApplicationVersion == "unknown":
             storage_dir_name = os.path.join(cls.ApplicationIdentifier, cls.ApplicationVersion)
         else:
-            from UM.Version import Version
             version = Version(cls.ApplicationVersion)
             storage_dir_name = os.path.join(cls.ApplicationIdentifier, "%s.%s" % (version.getMajor(), version.getMinor()))
 
@@ -500,7 +501,10 @@ class Resources:
         temp_dir_path = os.path.join(temp_root_dir_path, base_dir_name)
         # src -> temp -> dest
         try:
-            shutil.copytree(src_path, temp_dir_path, ignore = shutil.ignore_patterns("*.lock", "*.log", "old"))
+            # Copy everything, except for the logs, lock or really old (we used to copy old configs to the "old" folder)
+            # config files.
+            shutil.copytree(src_path, temp_dir_path,
+                            ignore = shutil.ignore_patterns("*.lock", "*.log", "*.log.?", "old"))
             # if the dest_path exist, it needs to be removed first
             if not os.path.exists(dest_path):
                 shutil.move(temp_dir_path, dest_path)
@@ -519,30 +523,41 @@ class Resources:
         }
         check_dir_type_func = check_dir_type_func_dict[dir_type]
 
-        latest_config_path = None
+        # CURA-6744
+        # If the application version matches "<major>.<minor>", create a Version object for it for comparison, so we
+        # can find the directory with the highest version that's below the application version.
+        # An application version that doesn't match "<major>.<minor>", e.g. "master", probably indicates a temporary
+        # version, and in this case, this temporary version is treated as the latest version. It will ONLY upgrade from
+        # a highest "<major>.<minor>" version if it's present.
+        # For app version, there can be extra version strings at the end. For comparison, we only want the
+        # "<major>.<minor>.<patch>" part. Here we use a regex to find that part in the app version string.
+        semantic_version_regex = re.compile(r"(^[0-9]+\.([0-9]+)*).*$")
+        app_version = None  # type: Optional[Version]
+        app_version_str = cls.ApplicationVersion
+        if app_version_str is not None:
+            result = semantic_version_regex.match(app_version_str)
+            if result is not None:
+                app_version_str = result.group(0)
+                app_version = Version(app_version_str)
+
+        latest_config_path = None  # type: Optional[str]
         for search_path in search_path_list:
             if not os.path.exists(search_path):
                 continue
 
             # Give priority to a folder with files with version number in it
             storage_dir_name_list = next(os.walk(search_path))[1]
-            if storage_dir_name_list:
-                storage_dir_name_list = sorted(storage_dir_name_list, reverse = True)
-                # for now we use alphabetically ordering to determine the latest version (excluding master)
-                for dir_name in storage_dir_name_list:
-                    if dir_name.endswith("master"):
-                        continue
-                    if version_regex.match(dir_name) is None:
-                        continue
 
-                    # make sure that the version we found is not newer than the current version
-                    if version_regex.match(cls.ApplicationVersion):
-                        later_version = sorted([cls.ApplicationVersion, dir_name], reverse = True)[0]
-                        if cls.ApplicationVersion != later_version:
-                            continue
+            match_dir_name_list = [n for n in storage_dir_name_list if version_regex.match(n) is not None]
+            match_dir_version_list = [{"dir_name": n, "version": Version(n)} for n in match_dir_name_list]  # type: List[Dict[str, Union[str, Version]]]
+            match_dir_version_list = sorted(match_dir_version_list, key = lambda x: x["version"], reverse = True)
+            if app_version is not None:
+                match_dir_version_list = list(x for x in match_dir_version_list if x["version"] < app_version)
+
+            if len(match_dir_version_list) > 0:
+                if isinstance(match_dir_version_list[0]["dir_name"], str):
+                    latest_config_path = os.path.join(search_path, match_dir_version_list[0]["dir_name"])  # type: ignore
 
-                    latest_config_path = os.path.join(search_path, dir_name)
-                    break
             if latest_config_path is not None:
                 break
 
