@@ -17,6 +17,7 @@ from UM.Signal import Signal
 if TYPE_CHECKING:
     from UM.Mesh.MeshData import MeshData
 
+
 ##  A SceneNode subclass that provides a camera object.
 #
 #   The camera provides a projection matrix and its transformation matrix
@@ -24,13 +25,13 @@ if TYPE_CHECKING:
 class Camera(SceneNode.SceneNode):
     class PerspectiveMode(enum.Enum):
         PERSPECTIVE = "perspective"
-        ORTHOGONAL = "orthogonal"
+        ORTHOGRAPHIC = "orthographic"
 
     @staticmethod
-    def getDefaultZoomFactor():
+    def getDefaultZoomFactor() -> float:
         return -0.3334
 
-    def __init__(self, name: str = "", parent: SceneNode.SceneNode = None) -> None:
+    def __init__(self, name: str = "", parent: Optional[SceneNode.SceneNode] = None) -> None:
         super().__init__(parent)
         self._name = name  # type: str
         self._projection_matrix = Matrix()  # type: Matrix
@@ -42,8 +43,9 @@ class Camera(SceneNode.SceneNode):
         self._window_height = 0  # type: int
         self._auto_adjust_view_port_size = True  # type: bool
         self.setCalculateBoundingBox(False)
-        self._cached_view_projection_matrix = None # type: Optional[Matrix]
-
+        self._cached_view_projection_matrix = None  # type: Optional[Matrix]
+        self._camera_light_position = None
+        self._cached_inversed_world_transformation = None
         self._zoom_factor = Camera.getDefaultZoomFactor()
 
         from UM.Application import Application
@@ -51,8 +53,6 @@ class Camera(SceneNode.SceneNode):
         Application.getInstance().getPreferences().preferenceChanged.connect(self._preferencesChanged)
         self._preferencesChanged("general/camera_perspective_mode")
 
-        self._cached_view_projection_matrix = None  # type: Optional[Matrix]
-
     def __deepcopy__(self, memo: Dict[int, object]) -> "Camera":
         copy = cast(Camera, super().__deepcopy__(memo))
         copy._projection_matrix = self._projection_matrix
@@ -66,7 +66,7 @@ class Camera(SceneNode.SceneNode):
         return self._zoom_factor
 
     def setZoomFactor(self, zoom_factor: float) -> None:
-        # Only an orthogonal camera has a zoom at the moment.
+        # Only an orthographic camera has a zoom at the moment.
         if not self.isPerspective():
             if self._zoom_factor != zoom_factor:
                 self._zoom_factor = zoom_factor
@@ -101,7 +101,7 @@ class Camera(SceneNode.SceneNode):
         self._viewport_height = height
         self._updatePerspectiveMatrix()
 
-    def _updatePerspectiveMatrix(self):
+    def _updatePerspectiveMatrix(self) -> None:
         view_width = self._viewport_width
         view_height = self._viewport_height
         projection_matrix = Matrix()
@@ -119,15 +119,17 @@ class Camera(SceneNode.SceneNode):
         self.setProjectionMatrix(projection_matrix)
         self.perspectiveChanged.emit(self)
 
-    def getViewProjectionMatrix(self):
+    def getViewProjectionMatrix(self) -> Matrix:
         if self._cached_view_projection_matrix is None:
             inverted_transformation = self.getWorldTransformation()
             inverted_transformation.invert()
             self._cached_view_projection_matrix = self._projection_matrix.multiply(inverted_transformation, copy = True)
         return self._cached_view_projection_matrix
 
-    def _updateWorldTransformation(self):
+    def _updateWorldTransformation(self) -> None:
         self._cached_view_projection_matrix = None
+        self._cached_inversed_world_transformation = None
+        self._camera_light_position = None
         super()._updateWorldTransformation()
     
     def getViewportHeight(self) -> int:
@@ -143,13 +145,24 @@ class Camera(SceneNode.SceneNode):
     def render(self, renderer) -> bool:
         # It's a camera. It doesn't need rendering.
         return True
-    
+
     ##  Set the projection matrix of this camera.
     #   \param matrix The projection matrix to use for this camera.
     def setProjectionMatrix(self, matrix: Matrix) -> None:
         self._projection_matrix = matrix
         self._cached_view_projection_matrix = None
 
+    def getInverseWorldTransformation(self):
+        if self._cached_inversed_world_transformation is None:
+            self._cached_inversed_world_transformation = self.getWorldTransformation()
+            self._cached_inversed_world_transformation.invert()
+        return self._cached_inversed_world_transformation
+
+    def getCameraLightPosition(self):
+        if self._camera_light_position is None:
+            self._camera_light_position = self.getWorldPosition() + Vector(0, 50, 0)
+        return self._camera_light_position
+
     def isPerspective(self) -> bool:
         return self._perspective
 
@@ -197,7 +210,7 @@ class Camera(SceneNode.SceneNode):
             origin = self.getWorldPosition()
             direction = -direction
         else:
-            # In orthogonal mode, the origin is the click position on the plane where the camera resides, and that
+            # In orthographic mode, the origin is the click position on the plane where the camera resides, and that
             # plane is parallel to the near and the far planes.
             projection = numpy.array([view_x, -view_y, 0.0, 1.0], dtype = numpy.float32)
             projection = numpy.dot(inverted_projection, projection)
@@ -219,15 +232,15 @@ class Camera(SceneNode.SceneNode):
         return position.x / position.z / 2.0, position.y / position.z / 2.0
 
     ##  Updates the _perspective field if the preference was modified.
-    def _preferencesChanged(self, key):
+    def _preferencesChanged(self, key: str) -> None:
         if key != "general/camera_perspective_mode":  # Only listen to camera_perspective_mode.
             return
         from UM.Application import Application
         new_mode = str(Application.getInstance().getPreferences().getValue("general/camera_perspective_mode"))
 
         # Translate the selected mode to the camera state.
-        if new_mode == str(self.PerspectiveMode.ORTHOGONAL.value):
-            Logger.log("d", "Changing perspective mode to orthogonal.")
+        if new_mode == str(self.PerspectiveMode.ORTHOGRAPHIC.value):
+            Logger.log("d", "Changing perspective mode to orthographic.")
             self.setPerspective(False)
         elif new_mode == str(self.PerspectiveMode.PERSPECTIVE.value):
             Logger.log("d", "Changing perspective mode to perspective.")
