@@ -477,7 +477,6 @@ class SceneNode:
         if not self._enabled or orientation == self._orientation:
             return
 
-
         if transform_space == SceneNode.TransformSpace.World:
             if self.getWorldOrientation() == orientation:
                 return
@@ -681,25 +680,18 @@ class SceneNode:
         for child in self._children:
             child._transformChanged()
 
-    def _updateLocalTransformation(self):
-        translation, euler_angle_matrix, scale, shear = self._transformation.decompose()
+    def _updateLocalTransformation(self) -> None:
+        self._position, euler_angle_matrix, self._scale, self._shear = self._transformation.decompose()
 
-        self._position = translation
-        self._scale = scale
-        self._shear = shear
-        orientation = Quaternion()
-        orientation.setByMatrix(euler_angle_matrix)
-        self._orientation = orientation
+        self._orientation.setByMatrix(euler_angle_matrix)
 
-    def _updateWorldTransformation(self):
+    def _updateWorldTransformation(self) -> None:
         if self._parent:
             self._world_transformation = self._parent.getWorldTransformation().multiply(self._transformation)
         else:
             self._world_transformation = self._transformation
 
-        world_translation, world_euler_angle_matrix, world_scale, world_shear = self._world_transformation.decompose()
-        self._derived_position = world_translation
-        self._derived_scale = world_scale
+        self._derived_position, world_euler_angle_matrix, self._derived_scale, world_shear = self._world_transformation.decompose()
         self._derived_orientation.setByMatrix(world_euler_angle_matrix)
 
     def _updateTransformation(self) -> None:
