@@ -52,7 +52,7 @@ MimeTypeDatabase.addMimeType(
 ##  A stack of setting containers to handle setting value retrieval.
 @signalemitter
 class ContainerStack(QObject, ContainerInterface, PluginObject):
-    Version = 4 # type: int
+    Version = 4  # type: int
 
     ##  Constructor
     #
@@ -241,15 +241,21 @@ class ContainerStack(QObject, ContainerInterface, PluginObject):
             if start_index >= len(self._containers):
                 return None
             containers = self._containers[start_index:]
-
-        for container in containers:
-            if skip_until_container and container.getId() != skip_until_container:
-                continue #Skip.
-            skip_until_container = None #When we find the container, stop skipping.
-
-            value = container.getProperty(key, property_name, context)
+        if property_name not in ["value", "state", "validationState"]:
+            # Value, state & validationState can be changed by instanceContainer, the rest cant. Ask the definition
+            # right away
+            value = containers[-1].getProperty(key, property_name, context)
             if value is not None:
                 return value
+        else:
+            for container in containers:
+                if skip_until_container and container.getId() != skip_until_container:
+                    continue #Skip.
+                skip_until_container = None #When we find the container, stop skipping.
+
+                value = container.getProperty(key, property_name, context)
+                if value is not None:
+                    return value
 
         if self._next_stack and use_next:
             return self._next_stack.getRawProperty(key, property_name, context = context,
