@@ -1,4 +1,4 @@
-# Copyright (c) 2018 Ultimaker B.V.
+# Copyright (c) 2020 Ultimaker B.V.
 # Uranium is released under the terms of the LGPLv3 or higher.
 
 import collections
@@ -6,6 +6,8 @@ import os.path
 from typing import List
 
 from PyQt5.QtCore import Qt, QAbstractListModel, QVariant, QModelIndex, QObject, pyqtProperty, pyqtSignal
+
+from UM.Decorators import deprecated
 from UM.FlameProfiler import pyqtSlot
 
 from UM.Logger import Logger
@@ -32,7 +34,7 @@ class SettingDefinitionsModel(QAbstractListModel):
     ExpandedRole = Qt.UserRole + 4
 
     def __init__(self, parent = None, *args, **kwargs):
-        super().__init__(parent = parent, *args, **kwargs)
+        super().__init__(parent = parent)
 
         self._container_id = None
         self._container = None
@@ -70,6 +72,8 @@ class SettingDefinitionsModel(QAbstractListModel):
 
         self.destroyed.connect(self._onDestroyed)
 
+        self.expandedChanged.connect(self.onExpandedChanged)
+
     ##  Emitted whenever the showAncestors property changes.
     showAncestorsChanged = pyqtSignal()
 
@@ -209,7 +213,7 @@ class SettingDefinitionsModel(QAbstractListModel):
             self.expandedChanged.emit()
             self._scheduleUpdateVisibleRows()
 
-    ##  Emitted whenever the exclude property changes
+    ##  Emitted whenever the expanded property changes
     expandedChanged = pyqtSignal()
 
     ##  This property indicates which settings should never be visibile.
@@ -260,13 +264,16 @@ class SettingDefinitionsModel(QAbstractListModel):
             self.expandedChanged.emit()
             self._scheduleUpdateVisibleRows()
 
+    def _getDefinitionsByKey(self, key: str) -> List["SettingDefinition"]:
+        if not self._container:
+            return []
+
+        return self._container.findDefinitions(key = key)
+
     ##  Show the children of a specified SettingDefinition and all children of those settings as well.
     @pyqtSlot(str)
     def expandRecursive(self, key: str) -> None:
-        if not self._container:
-            return
-
-        definitions = self._container.findDefinitions(key = key)
+        definitions = self._getDefinitionsByKey(key)
         if not definitions:
             return
         self.expand(key)
@@ -274,27 +281,33 @@ class SettingDefinitionsModel(QAbstractListModel):
         for child in definitions[0].children:
             self.expandRecursive(child.key)
 
-    ##  Hide the children of a specified SettingDefinition.
+    #@deprecated("Use collapseRecursive instead.", "4.5")  # Commented out because these two decorators don't work together.
     @pyqtSlot(str)
     def collapse(self, key: str) -> None:
-        if not self._container:
+        return self.collapseRecursive(key)
+
+    ##  Hide the children of a specified SettingDefinition and all children of those settings as well.
+    @pyqtSlot(str)
+    def collapseRecursive(self, key: str) -> None:
+        definitions = self._getDefinitionsByKey(key)
+        if not definitions:
             return
 
         if key not in self._expanded:
             return
 
-        definitions = self._container.findDefinitions(key = key)
-        if not definitions:
-            return
-
         self._expanded.remove(key)
 
         for child in definitions[0].children:
-            self.collapse(child.key)
+            self.collapseRecursive(child.key)
 
         self.expandedChanged.emit()
         self._scheduleUpdateVisibleRows()
 
+    @pyqtSlot()
+    def collapseAllCategories(self) -> None:
+        self.setExpanded([])
+
     ##  Show a single SettingDefinition.
     @pyqtSlot(str)
     def show(self, key: str) -> None:
@@ -343,9 +356,9 @@ class SettingDefinitionsModel(QAbstractListModel):
             # Ignore already hidden settings that need to be hidden.
             return
 
-        definitions = self._container.findDefinitions(key = key)
+        definitions = self._getDefinitionsByKey(key)
         if not definitions:
-            Logger.log("e", "Tried to change visiblity of a non-existant SettingDefinition")
+            Logger.log("e", "Tried to change visibility of a non-existent SettingDefinition")
             return
 
         if visible:
@@ -363,9 +376,7 @@ class SettingDefinitionsModel(QAbstractListModel):
 
     @pyqtSlot(str, result = int)
     def getIndex(self, key: str) -> int:
-        if not self._container:
-            return -1
-        definitions = self._container.findDefinitions(key = key)
+        definitions = self._getDefinitionsByKey(key)
         if not definitions:
             return -1
 
@@ -383,10 +394,7 @@ class SettingDefinitionsModel(QAbstractListModel):
 
     @pyqtSlot(str, str, result = "QVariantList")
     def getRequires(self, key, role = None):
-        if not self._container:
-            return []
-
-        definitions = self._container.findDefinitions(key = key)
+        definitions = self._getDefinitionsByKey(key)
         if not definitions:
             return []
 
@@ -408,10 +416,7 @@ class SettingDefinitionsModel(QAbstractListModel):
 
     @pyqtSlot(str, str, result = "QVariantList")
     def getRequiredBy(self, key, role = None):
-        if not self._container:
-            return []
-
-        definitions = self._container.findDefinitions(key = key)
+        definitions = self._getDefinitionsByKey(key)
         if not definitions:
             return []
 
@@ -527,6 +532,10 @@ class SettingDefinitionsModel(QAbstractListModel):
     def forceUpdate(self) -> None:
         self._update()
 
+    def onExpandedChanged(self):
+        for row in range(len(self._row_index_list)):
+            self.dataChanged.emit(self.index(row, 0), self.index(row, 0), [self.ExpandedRole])
+
     # Update the internal list of definitions and the visibility mapping.
     #
     # Note that this triggers a model reset and should only be called when the
