@@ -1,21 +1,22 @@
-# Copyright (c) 2016 Ultimaker B.V.
-# Uranium is released under the terms of the AGPLv3 or higher.
-
+# Copyright (c) 2019 Ultimaker B.V.
+# Uranium is released under the terms of the LGPLv3 or higher.
+from typing import Set
 from PyQt5.QtCore import QObject, pyqtSignal
 
+
 class SettingVisibilityHandler(QObject):
-    def __init__(self, parent = None, *args, **kwargs):
+    def __init__(self, parent = None, *args, **kwargs) -> None:
         super().__init__(parent = parent, *args, **kwargs)
 
-        self._visible = set()
+        self._visible = set()  # type: Set[str]
 
     visibilityChanged = pyqtSignal()
 
-    def setVisible(self, visible):
+    def setVisible(self, visible: Set[str]) -> None:
         if visible != self._visible:
             self._visible = visible
             self.visibilityChanged.emit()
 
-    def getVisible(self):
+    def getVisible(self) -> Set[str]:
         return self._visible.copy()
 
