@@ -17,7 +17,7 @@ import traceback
 import functools
 
 from UM.Event import CallFunctionEvent
-from UM.Decorators import deprecated, call_if_enabled
+from UM.Decorators import call_if_enabled
 from UM.Logger import Logger
 from UM.Platform import Platform
 from UM import FlameProfiler
@@ -71,7 +71,7 @@ class SignalQueue:
 # Integration with the Flame Profiler.
 
 
-def _recordSignalNames():
+def _recordSignalNames() -> bool:
     return FlameProfiler.enabled()
 
 
@@ -134,9 +134,9 @@ class Signal:
     #                 - type: The signal type. Defaults to Auto.
     def __init__(self, type: int = Auto) -> None:
         # These collections must be treated as immutable otherwise we lose thread safety.
-        self.__functions = WeakImmutableList()      # type: "WeakImmutableList"
-        self.__methods = WeakImmutablePairList()    # type: "WeakImmutablePairList"
-        self.__signals = WeakImmutableList()        # type: "WeakImmutableList"
+        self.__functions = WeakImmutableList()      # type: WeakImmutableList[Callable[[], None]]
+        self.__methods = WeakImmutablePairList()    # type: WeakImmutablePairList[Any, Callable[[], None]]
+        self.__signals = WeakImmutableList()        # type: WeakImmutableList[Signal]
 
         self.__lock = threading.Lock()  # Guards access to the fields above.
         self.__type = type
@@ -214,7 +214,7 @@ class Signal:
     ##  Connect to this signal.
     #   \param connector The signal or slot (function) to connect.
     @call_if_enabled(_traceConnect, _isTraceEnabled())
-    def connect(self, connector: Union['Signal', Callable[[],None]]) -> None:
+    def connect(self, connector: Union["Signal", Callable[[], None]]) -> None:
         if self._postpone_emit:
             Logger.log("w", "Tried to connect to signal %s that is currently being postponed, this is not possible", self.__name)
             return
@@ -296,7 +296,7 @@ class Signal:
 
     # Private implementation of the actual emit.
     # This is done to make it possible to freely push function events without needing to maintain state.
-    def __performEmit(self, *args, **kwargs):
+    def __performEmit(self, *args, **kwargs) -> None:
         # Quickly make some private references to the collections we need to process.
         # Although the these fields are always safe to use read and use with regards to threading,
         # we want to operate on a consistent snapshot of the whole set of fields.
@@ -341,8 +341,9 @@ class Signal:
     #     return "Signal<{}> {{ __functions={{ {} }}, __methods={{ {} }}, __signals={{ {} }} }}".format(id(self), function_str, method_str, signal_str)
 
 
-def strMethodSet(method_set):
-    return "{" + ", ".join([str(m) for m in method_set]) + "}"
+#def strMethodSet(method_set):
+#    return "{" + ", ".join([str(m) for m in method_set]) + "}"
+
 
 class CompressTechnique(enum.Enum):
     NoCompression = 0
@@ -400,25 +401,6 @@ def postponeSignals(*signals, compress: CompressTechnique = CompressTechnique.No
         signal._compress_postpone = False
 
 
-##  Convenience class to simplify signal creation.
-#
-#   This class is a Convenience class to simplify signal creation. Since signals
-#   need to be instance variables, normally you would need to create all signals
-#   in the class" `__init__` method. However, this makes them rather awkward to
-#   document. This class instead makes it possible to declare them as class variables,
-#   which makes documenting them near the function they are used possible.
-#   During the call to `__init__()`, this class will then search through all the
-#   properties of the instance and create instance variables for each class variable
-#   that is an instance of Signal.
-class SignalEmitter:
-    ##  Initialize method.
-    @deprecated("Please use the new @signalemitter decorator", "2.2")
-    def __init__(self, **kwargs):
-        super().__init__()
-        for name, signal in inspect.getmembers(self, lambda i: isinstance(i, Signal)):
-            setattr(self, name, Signal(type = signal.getType())) #pylint: disable=bad-whitespace
-
-
 ##  Class decorator that ensures a class has unique instances of signals.
 #
 #   Since signals need to be instance variables, normally you would need to create all
@@ -449,6 +431,7 @@ def signalemitter(cls):
     cls.__new__ = new_new
     return cls
 
+
 T = TypeVar('T')
 
 
@@ -458,7 +441,7 @@ T = TypeVar('T')
 #   it, but no application code can. Also, this class doesn't implement the Python
 #   list API, only the handful of methods we actually need in the code above.
 class WeakImmutableList(Generic[T], Iterable):
-    def __init__(self):
+    def __init__(self) -> None:
         self.__list = []    # type: List[ReferenceType[Optional[T]]]
 
     ## Append an item and return a new list
@@ -517,8 +500,8 @@ U = TypeVar('U')
 
 
 ##  A variation of WeakImmutableList which holds a pair of values using weak refernces.
-class WeakImmutablePairList(Generic[T,U], Iterable):
-    def __init__(self):
+class WeakImmutablePairList(Generic[T, U], Iterable):
+    def __init__(self) -> None:
         self.__list = []    # type: List[Tuple[ReferenceType[T],ReferenceType[U]]]
 
     ## Append an item and return a new list
@@ -576,4 +559,4 @@ class WeakImmutablePairListIterator:
             left = pair[0]()
             right = pair[1]()
 
-        return (left, right)
+        return left, right
