@@ -1,13 +1,18 @@
-# Copyright (c) 2019 Ultimaker B.V.
+# Copyright (c) 2020 Ultimaker B.V.
 # Uranium is released under the terms of the LGPLv3 or higher.
 
-from typing import Dict
+from typing import Dict, Optional, Tuple, Any
 from PyQt5.QtGui import QOpenGLVersionProfile, QOpenGLContext, QSurfaceFormat, QWindow
 
 from UM.Logger import Logger
+from UM.Platform import Platform
 
 
 class OpenGLContext:
+    class OpenGlVersionDetect:
+        Autodetect = "autodetect"
+        ForceLegacy = "force_legacy"
+        ForceModern = "force_modern"
 
     ##  Set OpenGL context, given major, minor version + core using QOpenGLContext
     #   Unfortunately, what you get back does not have to be the requested version.
@@ -101,11 +106,13 @@ class OpenGLContext:
     #   Note that PyQt only supports 4.1, 2.1 and 2.0. Cura omits support for 2.1, so the
     #   only returned options are 4.1 and 2.0.
     @classmethod
-    def detectBestOpenGLVersion(cls):
-        Logger.log("d", "Trying OpenGL context 4.1...")
-        ctx = cls.setContext(4, 1, core = True)
-        if ctx is not None:
-            fmt = ctx.format()
+    def detectBestOpenGLVersion(cls, force_compatability: bool) -> Tuple[Optional[int], Optional[int], Optional[int]]:
+        cls.detect_ogl_context = None
+        if not force_compatability:
+            Logger.log("d", "Trying OpenGL context 4.1...")
+            cls.detect_ogl_context = cls.setContext(4, 1, core = True)
+        if cls.detect_ogl_context is not None:
+            fmt = cls.detect_ogl_context.format()
             profile = fmt.profile()
 
             # First test: we hope for this
@@ -122,25 +129,22 @@ class OpenGLContext:
 
                 # CURA-6092: Check if we're not using software backed 4.1 context; A software 4.1 context
                 # is much slower than a hardware backed 2.0 context
+                # Check for OS, Since this only seems to happen on specific versions of Mac OSX and
+                # the workaround (which involves the deletion of an OpenGL context) is a problem for some Intel drivers.
+                if not Platform.isOSX():
+                    return major_version, minor_version, QSurfaceFormat.CoreProfile
+
                 gl_window = QWindow()
                 gl_window.setSurfaceType(QWindow.OpenGLSurface)
                 gl_window.showMinimized()
 
-                gl_format = QSurfaceFormat()
-                gl_format.setMajorVersion(major_version)
-                gl_format.setMinorVersion(minor_version)
-                gl_format.setProfile(profile)
-
-                gl_context = QOpenGLContext()
-                gl_context.setFormat(gl_format)
-                gl_context.create()
-                gl_context.makeCurrent(gl_window)
+                cls.detect_ogl_context.makeCurrent(gl_window)
 
                 gl_profile = QOpenGLVersionProfile()
                 gl_profile.setVersion(major_version, minor_version)
                 gl_profile.setProfile(profile)
 
-                gl = gl_context.versionFunctions(gl_profile) # type: Any #It's actually a protected class in PyQt that depends on the requested profile and the implementation of your graphics card.
+                gl = cls.detect_ogl_context.versionFunctions(gl_profile) # type: Any #It's actually a protected class in PyQt that depends on the requested profile and the implementation of your graphics card.
 
                 gpu_type = "Unknown"  # type: str
 
@@ -155,7 +159,7 @@ class OpenGLContext:
                     # Some Intel GPU chipsets return a string, which is not undecodable via PyQt5.
                     # This workaround makes the code fall back to a "Unknown" renderer in these cases.
                     try:
-                        gpu_type = gl.glGetString(gl.GL_RENDERER) #type: str
+                        gpu_type = gl.glGetString(gl.GL_RENDERER)
                     except UnicodeDecodeError:
                         Logger.log("e", "DecodeError while getting GL_RENDERER via glGetString!")
 
@@ -169,9 +173,9 @@ class OpenGLContext:
 
         # Fallback: check min spec
         Logger.log("d", "Trying OpenGL context 2.0...")
-        ctx = cls.setContext(2, 0, profile = QSurfaceFormat.NoProfile)
-        if ctx is not None:
-            fmt = ctx.format()
+        cls.detect_ogl_context = cls.setContext(2, 0, profile = QSurfaceFormat.NoProfile)
+        if cls.detect_ogl_context is not None:
+            fmt = cls.detect_ogl_context.format()
             profile = fmt.profile()
 
             if fmt.majorVersion() >= 2 and fmt.minorVersion() >= 0:
@@ -206,5 +210,8 @@ class OpenGLContext:
     minor_version = 0
     profile = None  # type: QSurfaceFormat
 
+    # Keep already created context in memory, as some drivers (Intel) have trouble deleting OpenGL-contexts:
+    detect_ogl_context = None  #type: Optional[QOpenGLContext]
+
     # To be filled by helper functions
     properties = {}  # type: Dict[str, bool]
