@@ -4,7 +4,7 @@
 import configparser
 import ast
 
-from PyQt5.QtGui import QOpenGLShader, QOpenGLShaderProgram, QVector2D, QVector3D, QVector4D, QMatrix4x4, QColor, QImage, QOpenGLTexture, QOpenGLVertexArrayObject, QOpenGLBuffer
+from PyQt5.QtGui import QOpenGLShader, QOpenGLShaderProgram, QVector2D, QVector3D, QVector4D, QMatrix4x4, QColor
 from UM.Logger import Logger
 
 from UM.Math.Vector import Vector
@@ -16,6 +16,7 @@ from UM.Math.Color import Color
 class InvalidShaderProgramError(Exception):
     pass
 
+
 ##  An abstract class for dealing with shader programs.
 #
 #   This class provides an interface an some basic elements for dealing with
@@ -169,6 +170,7 @@ class ShaderProgram:
 
         if self._bound:
             self._setUniformValueDirect(uniform, value)
+
     ##  Set a texture that should be bound to a specified texture unit when this shader is bound.
     #
     #   \param texture_unit \type{int} The texture unit to bind the texture to.
@@ -188,7 +190,7 @@ class ShaderProgram:
     #   \param stride The stride of the attribute.
     #
     #   \note If the shader is not bound, this will bind the shader.
-    def enableAttribute(self, name, type, offset, stride = 0):
+    def enableAttribute(self, name: str, type: str, offset: int, stride: int = 0) -> None:
         if not self._shader_program:
             return
 
@@ -201,15 +203,15 @@ class ShaderProgram:
         if attribute == -1:
             return
 
-        if type is "int":
+        if type == "int":
             self._shader_program.setAttributeBuffer(attribute, 0x1404, offset, 1, stride) #GL_INT
-        elif type is "float":
+        elif type == "float":
             self._shader_program.setAttributeBuffer(attribute, 0x1406, offset, 1, stride) #GL_FLOAT
-        elif type is "vector2f":
+        elif type == "vector2f":
             self._shader_program.setAttributeBuffer(attribute, 0x1406, offset, 2, stride) #GL_FLOAT
-        elif type is "vector3f":
+        elif type == "vector3f":
             self._shader_program.setAttributeBuffer(attribute, 0x1406, offset, 3, stride) #GL_FLOAT
-        elif type is "vector4f":
+        elif type == "vector4f":
             self._shader_program.setAttributeBuffer(attribute, 0x1406, offset, 4, stride) #GL_FLOAT
 
         self._shader_program.enableAttributeArray(attribute)
@@ -217,7 +219,7 @@ class ShaderProgram:
     ##  Disable a vertex attribute so it is no longer used.
     #
     #   \param name The name of the attribute to use.
-    def disableAttribute(self, name):
+    def disableAttribute(self, name: str) -> None:
         if not self._shader_program:
             return
 
@@ -227,7 +229,7 @@ class ShaderProgram:
         self._shader_program.disableAttributeArray(self._attribute_indices[name])
 
     ##  Bind the shader to use it for rendering.
-    def bind(self):
+    def bind(self) -> None:
         if not self._shader_program or not self._shader_program.isLinked():
             return
 
@@ -244,7 +246,7 @@ class ShaderProgram:
             texture.bind(texture_unit)
 
     ##  Release the shader so it will no longer be used for rendering.
-    def release(self):
+    def release(self) -> None:
         if not self._shader_program or not self._bound:
             return
 
@@ -269,7 +271,7 @@ class ShaderProgram:
     ##  Remove a uniform value binding.
     #
     #   \param key The uniform to remove.
-    def removeBinding(self, key):
+    def removeBinding(self, key: str) -> None:
         if key not in self._bindings:
             return
 
@@ -283,7 +285,7 @@ class ShaderProgram:
     #
     #   \note By default, these values are not cached as they are expected to be continuously
     #         updated.
-    def updateBindings(self, **kwargs):
+    def updateBindings(self, **kwargs) -> None:
         for key, value in kwargs.items():
             if key in self._bindings and value is not None:
                 self.setUniformValue(self._bindings[key], value, cache = False)
@@ -305,15 +307,14 @@ class ShaderProgram:
     ##  Remove an attribute binding.
     #
     #   \param key The name of the attribute binding to remove.
-    def removeAttributeBinding(self, key):
+    def removeAttributeBinding(self, key: str) -> None:
         if key not in self._attribute_bindings:
             return
 
         del self._attribute_bindings[key]
 
     def _matrixToQMatrix4x4(self, m):
-        return QMatrix4x4(m.at(0, 0), m.at(0, 1), m.at(0, 2), m.at(0, 3), m.at(1, 0), m.at(1, 1), m.at(1, 2), m.at(1, 3),
-            m.at(2, 0), m.at(2, 1), m.at(2, 2), m.at(2, 3), m.at(3, 0), m.at(3, 1), m.at(3, 2), m.at(3, 3))
+        return QMatrix4x4(m.getData().flatten())
 
     def _setUniformValueDirect(self, uniform, value):
         if type(value) is Vector:
