@@ -1,16 +1,21 @@
 # Copyright (c) 2019 Ultimaker B.V.
 # Uranium is released under the terms of the LGPLv3 or higher.
+from typing import List, Dict, Union, Optional, Any
 
 from UM.Logger import Logger
 from UM.Math.Matrix import Matrix
 
 from UM.Math.Vector import Vector
+from UM.Mesh.MeshData import MeshData
+from UM.Scene.Camera import Camera
 
 from UM.View.GL.OpenGL import OpenGL
 from UM.View.GL.OpenGLContext import OpenGLContext
 
 from PyQt5.QtGui import QOpenGLVertexArrayObject
 
+from UM.View.GL.ShaderProgram import ShaderProgram
+
 vertexBufferProperty = "__gl_vertex_buffer"
 indexBufferProperty = "__gl_index_buffer"
 
@@ -28,7 +33,7 @@ indexBufferProperty = "__gl_index_buffer"
 #   RenderBatch-changes. Whenever (sets of) RenderBatches are managed throughout
 #   the lifetime of a session, crossing multiple frames, the usage of VAO's can
 #   improve performance by reusing them.
-class RenderBatch():
+class RenderBatch:
     ##  The type of render batch.
     #
     #   This determines some basic state values, like blending on/off and additionally
@@ -70,23 +75,22 @@ class RenderBatch():
     #                                         This can be used to do additional alterations to the state that can not be done otherwise.
     #                                         The callback is passed the OpenGL bindings object as first and only parameter.
     #                 - state_teardown_callback: A callback similar to state_setup_callback, but called after everything was rendered, to handle cleaning up state changes made in state_setup_callback.
-    def __init__(self, shader, **kwargs):
+    def __init__(self, shader: ShaderProgram, **kwargs) -> None:
         self._shader = shader
-        self._render_type = kwargs.get("type", self.RenderType.Solid)
-        self._render_mode = kwargs.get("mode", self.RenderMode.Triangles)
-        self._backface_cull = kwargs.get("backface_cull", False)
+        self._render_type = kwargs.get("type", self.RenderType.Solid)  # type: int
+        self._render_mode = kwargs.get("mode", self.RenderMode.Triangles)  # type: int
+        self._backface_cull = kwargs.get("backface_cull", False)  # type: bool
         self._render_range = kwargs.get("range", None)
-        self._sort_weight = kwargs.get("sort", 0)
+        self._sort_weight = kwargs.get("sort", 0)  # type: int
         self._blend_mode = kwargs.get("blend_mode", None)
         if not self._blend_mode:
             self._blend_mode = self.BlendMode.NoBlending if self._render_type == self.RenderType.Solid else self.BlendMode.Normal
         self._state_setup_callback = kwargs.get("state_setup_callback", None)
         self._state_teardown_callback = kwargs.get("state_teardown_callback", None)
-        self._items = []
+        self._items = []  # type: List[Dict[str, Union[MeshData, Matrix, Dict[str, Any]]]]
 
-        self._view_matrix = None
-        self._projection_matrix = None
-        self._view_projection_matrix = None
+        self._view_matrix = None  # type: Optional[Matrix]
+        self._projection_matrix = None  # type: Optional[Matrix]
 
         self._gl = OpenGL.getInstance().getBindingsObject()
 
@@ -146,7 +150,7 @@ class RenderBatch():
     #   \param mesh The mesh to render with the transform matrix.
     #   \param uniforms A dict of additional uniform bindings to set when rendering the item.
     #                   Note these are set specifically for this item.
-    def addItem(self, transformation, mesh, uniforms = None):
+    def addItem(self, transformation: Optional[Matrix], mesh: Optional[MeshData], uniforms = None):
         if not transformation:
             Logger.log("w", "Tried to add an item to batch without transformation")
             return
@@ -159,7 +163,7 @@ class RenderBatch():
     ##  Render the batch.
     #
     #   \param camera The camera to render from.
-    def render(self, camera):
+    def render(self, camera: Optional[Camera]):
         if camera is None:
             Logger.log("e", "Unable to render batch without a camera.")
             return
@@ -192,17 +196,15 @@ class RenderBatch():
         if self._state_setup_callback:
             self._state_setup_callback(self._gl)
 
-        self._view_matrix = camera.getWorldTransformation()
-        self._view_matrix.invert()
+        self._view_matrix = camera.getInverseWorldTransformation()
+
         self._projection_matrix = camera.getProjectionMatrix()
-        self._view_projection_matrix = camera.getViewProjectionMatrix()
 
         self._shader.updateBindings(
             view_matrix = self._view_matrix,
             projection_matrix = self._projection_matrix,
-            view_projection_matrix = self._view_projection_matrix,
             view_position = camera.getWorldPosition(),
-            light_0_position = camera.getWorldPosition() + Vector(0, 50, 0)
+            light_0_position = camera.getCameraLightPosition()
         )
 
         # The VertexArrayObject (VAO) works like a VCR, recording buffer activities in the GPU.
@@ -213,7 +215,8 @@ class RenderBatch():
             vao.create()
             if not vao.isCreated():
                 Logger.log("e", "VAO not created. Hell breaks loose")
-            vao.bind()
+            else:
+                vao.bind()
 
         for item in self._items:
             self._renderItem(item)
@@ -223,7 +226,7 @@ class RenderBatch():
 
         self._shader.release()
 
-    def _renderItem(self, item):
+    def _renderItem(self, item: Dict[str, Any]):
         transformation = item["transformation"]
         mesh = item["mesh"]
 
@@ -239,14 +242,9 @@ class RenderBatch():
             normal_matrix.invert()
             normal_matrix.transpose()
 
-        model_view_matrix = transformation.preMultiply(self._view_matrix, copy = True)
-        model_view_projection_matrix = transformation.preMultiply(self._view_projection_matrix, copy = True)
-
         self._shader.updateBindings(
             model_matrix = transformation,
-            normal_matrix = normal_matrix,
-            model_view_matrix = model_view_matrix,
-            model_view_projection_matrix = model_view_projection_matrix
+            normal_matrix = normal_matrix
         )
 
         if item["uniforms"] is not None:
@@ -261,7 +259,7 @@ class RenderBatch():
             # glDrawRangeElements does not work as expected and did not get the indices field working..
             # Now we're just uploading a clipped part of the array and the start index always becomes 0.
             index_buffer = OpenGL.getInstance().createIndexBuffer(
-                mesh, force_recreate = True, index_start = self._render_range[0], index_stop = self._render_range[1])
+                mesh, force_recreate=True, index_start = self._render_range[0], index_stop = self._render_range[1])
         if index_buffer is not None:
             index_buffer.bind()
 
