@@ -1,6 +1,8 @@
 # Copyright (c) 2018 Ultimaker B.V.
 # Uranium is released under the terms of the LGPLv3 or higher.
+from typing import Optional
 
+from UM.Scene.SceneNode import SceneNode
 from UM.Tool import Tool
 from UM.Job import Job
 from UM.Event import Event, MouseEvent, KeyEvent
@@ -22,8 +24,10 @@ from UM.Operations.LayFlatOperation import LayFlatOperation
 from UM.Version import Version
 
 from UM.View.GL.OpenGL import OpenGL
-
-from . import RotateToolHandle
+try:
+    from . import RotateToolHandle
+except (ImportError, SystemError):
+    import RotateToolHandle  # type: ignore  # This fixes the tests not being able to import.
 
 import math
 import time
@@ -31,10 +35,10 @@ import time
 from UM.i18n import i18nCatalog
 i18n_catalog = i18nCatalog("uranium")
 
+
 ##  Provides the tool to rotate meshes and groups
 #
 #   The tool exposes a ToolHint to show the rotation angle of the current operation
-
 class RotateTool(Tool):
     def __init__(self):
         super().__init__()
@@ -52,9 +56,10 @@ class RotateTool(Tool):
         self._iterations = 0
         self._total_iterations = 0
         self._rotating = False
-        self.setExposedProperties("ToolHint", "RotationSnap", "RotationSnapAngle", "SelectFaceSupported")
+        self.setExposedProperties("ToolHint", "RotationSnap", "RotationSnapAngle", "SelectFaceSupported", "SelectFaceToLayFlatMode")
         self._saved_node_positions = []
 
+        self._select_face_mode = False
         Selection.selectedFaceChanged.connect(self._onSelectedFaceChanged)
 
     ##  Handle mouse and keyboard events
@@ -186,6 +191,9 @@ class RotateTool(Tool):
                 return True
 
     def _onSelectedFaceChanged(self):
+        if not self._select_face_mode:
+            return
+
         self._handle.setEnabled(not Selection.getFaceSelectMode())
 
         selected_face = Selection.getSelectedFace()
@@ -196,6 +204,8 @@ class RotateTool(Tool):
         meshdata = original_node.getMeshDataTransformed()
         if not meshdata or face_id < 0:
             return
+        if face_id > (meshdata.getVertexCount() / 3 if not meshdata.hasIndices() else meshdata.getFaceCount()):
+            return
 
         face_mid, face_normal = meshdata.getFacePlane(face_id)
         object_mid = original_node.getBoundingBox().center
@@ -260,9 +270,27 @@ class RotateTool(Tool):
             self._snap_angle = angle
             self.propertyChanged.emit()
 
+    ##  Wether the rotate tool is in 'Lay flat by face'-Mode.
+    #
+    #   \return (bool)
+    def getSelectFaceToLayFlatMode(self) -> bool:
+        if not Selection.getFaceSelectMode():
+            self._select_face_mode = False  # .. but not the other way around!
+        return self._select_face_mode
+
+    ##  Set the rotate tool to/from 'Lay flat by face'-Mode.
+    #
+    #   \param (bool)
+    def setSelectFaceToLayFlatMode(self, select: bool) -> None:
+        if select != self._select_face_mode or select != Selection.getFaceSelectMode():
+            self._select_face_mode = select
+            if not select:
+                Selection.clearFace()
+            Selection.setFaceSelectMode(self._select_face_mode)
+            self.propertyChanged.emit()
+
     ##  Reset the orientation of the mesh(es) to their original orientation(s)
     def resetRotation(self):
-
         for node in self._getSelectedObjectsWithoutSelectedAncestors():
             node.setMirror(Vector(1, 1, 1))
 
